import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Font;
import java.awt.Graphics2D;
import java.awt.Image;
import java.awt.MediaTracker;
import java.awt.Toolkit;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import javax.swing.JFrame;

public class GUI extends JFrame implements MouseListener {

	Algorithmus algorithmus;
	Graphics2D graphics;

	public GUI() {
		this.addMouseListener(this);
		this.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		this.setTitle("Dijkstra-Algorithmus");
		this.initialisiere();
	}

	public void initialisiere() {
		algorithmus = new Algorithmus();
		this.setVisible(true);
		this.setSize(634, 769);
		this.graphics = (Graphics2D) this.getContentPane().getGraphics();
		Image img = Toolkit.getDefaultToolkit().getImage("Lageplan.gif");
		MediaTracker mediaTracker = new MediaTracker(this);
		mediaTracker.addImage(img, 0);
		try {
			mediaTracker.waitForID(0);
		} catch (InterruptedException ie) {
			System.err.println(ie);
			System.exit(1);
		}
		graphics.drawImage(img, 0, 0, null);
	}

	/**
	 * findet den Knoten, der am nchsten an dem angegebenen Punkt liegt. 
	 */
	public Knoten findeNaechstenKnoten(int mouseX, int mouseY) {
		Knoten knoten[] = algorithmus.graph;
		int zaehler = 1;
		Knoten aktuellerKnoten = knoten[zaehler];
		Knoten naechsterKnoten = aktuellerKnoten;
		int aktuelleDistanz = (int) naechsterKnoten.berechneMausDistanz(mouseX, mouseY);
		zaehler++;
		while (zaehler < 19) {
			aktuellerKnoten = knoten[zaehler];
			if (aktuellerKnoten.berechneMausDistanz(mouseX, mouseY) < aktuelleDistanz) {
				naechsterKnoten = aktuellerKnoten;
				aktuelleDistanz = (int) aktuellerKnoten.berechneMausDistanz(mouseX, mouseY);
			}
			zaehler++;
		}
		return naechsterKnoten;
	}

	public void zeichneKnoten(Knoten k, Color farbe) {
		graphics.setColor(farbe);
		graphics.fillOval(k.x - 18, k.y - 18, 37, 37);
		graphics.setColor(Color.black);
		graphics.setStroke(new BasicStroke(2));
		graphics.drawOval(k.x - 18, k.y - 18, 37, 37);
		graphics.setFont(new Font("Helvetica", Font.PLAIN, 24));
		graphics.drawString(k.name, k.x - 7, k.y + 9);
	}

	public void zeichneWeg(Knoten knoten, Color farbe) {
		zeichneKnoten(knoten, farbe);
		if (knoten.vorgaengerImPfad != null) {
			zeichneWeg(knoten.vorgaengerImPfad, farbe);
		}
	}

	// Methoden des Mouse-Listener
	
	public void mouseClicked(MouseEvent e) {
		if (algorithmus.startknoten == null && algorithmus.zielknoten == null) {
			algorithmus.startknoten = this.findeNaechstenKnoten(e.getX(), e.getY());
			this.zeichneKnoten(algorithmus.startknoten, Color.red);
		} else if (algorithmus.startknoten != null && algorithmus.zielknoten == null) {
			algorithmus.zielknoten = this.findeNaechstenKnoten(e.getX(), e.getY());
			this.zeichneKnoten(algorithmus.zielknoten, Color.green);
			algorithmus.berechneWeg();
			this.zeichneWeg(algorithmus.zielknoten, Color.blue);
		} else {
			algorithmus.zielknoten = null;
			algorithmus.startknoten = null;
			this.initialisiere();
		}
	}

	public void mouseEntered(MouseEvent e) {
	}

	public void mouseExited(MouseEvent e) {
	}

	public void mousePressed(MouseEvent e) {
	}

	public void mouseReleased(MouseEvent e) {
	}

	public static void main(String[] argv) {
		GUI ui = new GUI();
		ui.initialisiere();
	}

}